#include <math.h>
#include <allegro.h>
#include "alleggl.h"
#include "triangles.h"
#include "map.h"
#include "drawing.h"

struct TILECOOLDOWN		*cooldown;
struct MAPPOS			mapdata[20][20];
struct OBJECTINFO		*objectinfo;
struct SMALLMAPITEM		*smallmap[200][200];

struct OBJECTINFO		*shadow_object;

void draw_objects(void);
void draw_tiles(double x, double y);
double fall_down(double x, double y, double z, TILEPOS **tileinfo);
void add_tilecooldown(int timer, TILEPOS *tileinfo);
void check_cooldown(void);
void clear_map(void);
void free_map(void);
void add_tile(double x, double y, double z, struct TRIANGLEINFO *tiledata, int tile_id);
void add_object(double x, double y, double z, TRIANGLEINFO *objectdata, int object_id, int size);
void free_objects(void);
OBJECTINFO *intersects_with(double x, double y, double z);
int direction_possible(double x, double y, double z);
int direction_falldown(double x, double y, double z);
int direction_falldead(double x, double y, double z);

OBJECTINFO *intersects_with(double x, double y, double z)
{
	struct OBJECTINFO *currobject, *foundobject;

	foundobject=NULL;
	currobject=objectinfo;
	while (currobject!=NULL)
	{
		if (currobject->object_id>=0)
		{
			if (x>=currobject->x-currobject->size && x<=currobject->x+currobject->size &&
				y>=currobject->y-currobject->size && y<=currobject->y+currobject->size &&
				z>=currobject->z-4 && z<=currobject->z+4)
				foundobject=currobject;
		}
		currobject=currobject->nextobject;
	}
	return foundobject;
}

void draw_objects(void)
{
	struct OBJECTINFO *currobject;

	currobject=objectinfo;
	while (currobject!=NULL)
	{
		if (currobject->object_id>=0)
			draw_object(currobject->x,currobject->y,currobject->z,
				currobject->objectdata,currobject->scalefac, currobject->rotate);
		currobject=currobject->nextobject;
	}
}

void add_object(double x, double y, double z, TRIANGLEINFO *objectdata, int object_id, int size)
{
	struct OBJECTINFO *newobject;

	newobject=(OBJECTINFO*)malloc(sizeof(OBJECTINFO));
	newobject->x=x;
	newobject->y=y;
	newobject->z=z;
	newobject->object_id=object_id;
	newobject->objectdata=objectdata;
	newobject->size=size;
	newobject->scalefac=1;
	newobject->rotate=-1;

	newobject->nextobject=objectinfo;
	objectinfo=newobject;
}

void free_objects(void)
{
	struct OBJECTINFO *currobject, *deleteobject;

	currobject=objectinfo;
	while (currobject!=NULL)
	{
		deleteobject=currobject;
		currobject=currobject->nextobject;
		free(deleteobject);
	}
	objectinfo=NULL;
}

void draw_tiles(double x, double y)
{
	int mapx, mapy, i1, i2;
	TILEPOS *tile;

	mapx=(int)(x+500)/50;
	mapy=(int)(y+500)/50;

	for (i1=mapx-1; i1<=mapx+1; i1++)
	if (i1>=0 && i1<=19)
		for (i2=mapy-1; i2<=mapy+1; i2++)
		if (i2>=0 && i2<=19)
		{
			tile=mapdata[i1][i2].tilelist;
			while (tile!=NULL)
			{
				draw_floortile(tile->x,tile->y,tile->z,tile->tiledata);
				tile=tile->nexttile;
			}
		}
}

double fall_down(double x, double y, double z, TILEPOS **tileinfo)
{
	int mapx, mapy;
	TILEPOS *tile;
	double falltoz;

	falltoz=-25;
	mapx=(int)(x+500)/50;
	mapy=(int)(y+500)/50;

	if (mapx>=0 && mapy>=0 && mapx<20 && mapy<20)
	{
		tile=mapdata[mapx][mapy].tilelist;
		while (tile!=NULL)
		{
			if (tile->z <= z)
			{
				if (x>=tile->x-4.5 && x<=tile->x+4.5 &&
					y>=tile->y-4.5 && y<=tile->y+4.5)
				{
					if (tile->z>falltoz)
					{
						falltoz=tile->z;
						*tileinfo=tile;
					}
				}
			}
			tile=tile->nexttile;
		}
	}

	return falltoz;
}

void add_tilecooldown(int timer, TILEPOS *tileinfo)
{
	TILECOOLDOWN *newcooldown;
	int found;

	found=FALSE;
	newcooldown=cooldown;
	while (newcooldown!=NULL && found==FALSE)
	{
		if (newcooldown->tileinfo==tileinfo)
			newcooldown->timer=0;
		if (newcooldown->timer<=0)
		{
			found=TRUE;
			newcooldown->timer=timer;
			newcooldown->initialtimer=timer;
			newcooldown->tileinfo=tileinfo;
		}
		else
			newcooldown=newcooldown->next;
	}

	if (found==FALSE)
	{
		newcooldown=(TILECOOLDOWN*)malloc(sizeof(TILECOOLDOWN));
		newcooldown->timer=timer;
		newcooldown->initialtimer=timer;
		newcooldown->tileinfo=tileinfo;
		newcooldown->next=cooldown;
		cooldown=newcooldown;
	}
}

void check_cooldown(void)
{
	TILECOOLDOWN *currcooldown;
	int i;

	currcooldown=cooldown;
	while (currcooldown!=NULL)
	{
		if (currcooldown->timer>0)
		{
			currcooldown->timer--;
			if (currcooldown->timer<=0)
			{
				if (currcooldown->tileinfo->tile_id==20)
				{
					// restore trampoline
					for (i=1; i<=3; i++)
						if (currcooldown->tileinfo->tiledata==trampoline[i])
							currcooldown->tileinfo->tiledata=trampoline[i-1];
					if (currcooldown->tileinfo->tiledata==NULL)
						currcooldown->tileinfo->tiledata=trampoline[3];
					
					if (currcooldown->tileinfo->tiledata!=trampoline[0])
						currcooldown->timer=currcooldown->initialtimer;
				}

			}
		}
		currcooldown=currcooldown->next;
	}
}

void clear_map(void)
{
	int x,y;
	for (y=0; y<20; y++)
		for (x=0; x<20; x++)
			mapdata[x][y].tilelist=NULL;
	for (y=0; y<200; y++)
		for (x=0; x<200; x++)
			smallmap[x][y]=NULL;
}

void free_map(void)
{
	int x,y;
	TILEPOS *tile, *deletetile;
	SMALLMAPITEM *mapinfo, *deletemapinfo;

	for (y=0; y<20; y++)
		for (x=0; x<20; x++)
		{
			tile=mapdata[x][y].tilelist;
			while (tile!=NULL)
			{
				deletetile=tile;
				tile=tile->nexttile;
				free(deletetile);
			}
			mapdata[x][y].tilelist=NULL;
		}

	for (y=0; y<200; y++)
		for (x=0; x<200; x++)
		{
			mapinfo=smallmap[x][y];
			while (mapinfo!=NULL)
			{
				deletemapinfo=mapinfo;
				mapinfo=mapinfo->next;
				free(deletemapinfo);
			}
			smallmap[x][y]=NULL;
		}
}

void add_tile(double x, double y, double z, struct TRIANGLEINFO *tiledata, int tile_id)
{
	int mapx, mapy;
	TILEPOS *newtile;
	SMALLMAPITEM *newmapinfo;

	mapx=(int)(x+500)/50;
	mapy=(int)(y+500)/50;
	if (mapx<0) mapx=0;
	if (mapy<0) mapy=0;
	if (mapx>19) mapx=19;
	if (mapy>19) mapy=19;
	
	newtile=(TILEPOS*)malloc(sizeof(TILEPOS));
	newtile->nexttile=mapdata[mapx][mapy].tilelist;
	newtile->x=x;
	newtile->y=y;
	newtile->z=z;
	newtile->tiledata=tiledata;
	newtile->tile_id=tile_id;
	mapdata[mapx][mapy].tilelist=newtile;


	mapx=(int)(x+500)/5;
	mapy=(int)(y+500)/5;
	if (mapx<0) mapx=0;
	if (mapy<0) mapy=0;
	if (mapx>199) mapx=199;
	if (mapy>199) mapy=199;
	newmapinfo=(SMALLMAPITEM*)malloc(sizeof(SMALLMAPITEM));
	newmapinfo->next=smallmap[mapx][mapy];
	newmapinfo->height=z;
	newmapinfo->tile_id=tile_id;
	smallmap[mapx][mapy]=newmapinfo;
}

int direction_possible(double x, double y, double z)
{
	int possible;
	int mapx, mapy;
	SMALLMAPITEM *mapinfo;
	
	possible=TRUE;
	mapx=(int)(x+500)/5;
	mapy=(int)(y+500)/5;
	if (mapx<0) mapx=0;
	if (mapy<0) mapy=0;
	if (mapx>199) mapx=199;
	if (mapy>199) mapy=199;
	mapinfo=smallmap[mapx][mapy];
	while (mapinfo!=NULL)
	{
		switch (mapinfo->tile_id)
		{
		case 1: // tile with walls
				if (mapinfo->height>z)
					possible=FALSE;
			break;
		case 30: //
				if (z<mapinfo->height && z>mapinfo->height-20)
					possible=FALSE;
			break;
		}
		mapinfo=mapinfo->next;
	}

	return possible;
}

int direction_falldown(double x, double y, double z)
{
	int falldown;
	int mapx, mapy;
	SMALLMAPITEM *mapinfo;
	
	falldown=FALSE;
	mapx=(int)(x+500)/5;
	mapy=(int)(y+500)/5;
	if (mapx<0) mapx=0;
	if (mapy<0) mapy=0;
	if (mapx>199) mapx=199;
	if (mapy>199) mapy=199;
	mapinfo=smallmap[mapx][mapy];
	while (mapinfo!=NULL)
	{
		if (mapinfo->height<z)
			falldown=TRUE;
		mapinfo=mapinfo->next;
	}

	return falldown;
}

int direction_falldead(double x, double y, double z)
{
	int falldead;
	int mapx, mapy;
	SMALLMAPITEM *mapinfo;
	
	falldead=TRUE;
	mapx=(int)(x+500)/5;
	mapy=(int)(y+500)/5;
	if (mapx<0) mapx=0;
	if (mapy<0) mapy=0;
	if (mapx>199) mapx=199;
	if (mapy>199) mapy=199;
	mapinfo=smallmap[mapx][mapy];
	while (mapinfo!=NULL)
	{
		if (mapinfo->height<=z)
			falldead=FALSE;
		mapinfo=mapinfo->next;
	}

	return falldead;
}
