#define USE_MOD_PLAY

#include <allegro.h>
#include <math.h>
#include "utils.h"
#include "sound.h"
#ifdef USE_MOD_PLAY
	#include <jgmod.h>
#endif

struct SOUNDBLOCK
{ int                   length;
  int                   pos;
  double                freq;
  double                freqinc;
  double                ampl;
  double                amplinc;
  int                   fadein_len;
  int                   fadeout_len;
  int                   wave_type;
  double                sinpos;
  int                   strength;
} soundblock;

int						music_on=TRUE;
int						sound_on=TRUE;
SAMPLE					*sounds[16];
int                     voice_nr[16];
#ifdef USE_MOD_PLAY
	JGMOD					*music;
#endif

#include "sound.h"

void install_sound(void)
{
	reserve_voices (8, -1);
    if (install_sound (DIGI_AUTODETECT, MIDI_NONE, NULL) < 0) 
		{ music_on=FALSE; sound_on=FALSE; }
#ifdef USE_MOD_PLAY
    if (install_mod (4) < 0) 
		music_on=FALSE;
	if (music_on)
		set_mod_volume (50);
#endif
}

void load_music(void)
{
#ifdef USE_MOD_PLAY
	if (music_on)
	{
		if (file_exists("supox.mod", FA_ARCH, NULL))
			music=load_mod("supox.mod"); 
		else
			music_on=FALSE;
	}
#endif
}

void start_music(void)
{
#ifdef USE_MOD_PLAY
	if (music_on)
		play_mod (music, TRUE);
#endif
}

void stop_music(void)
{
#ifdef USE_MOD_PLAY
	if (music_on)
		stop_mod();
#endif
}

void destroy_music(void)
{
#ifdef USE_MOD_PLAY
	if (music_on)
		destroy_mod(music);
#endif
}

void create_samples(void)
{	int i;

// [wave_type,length,start_freq,end_freq,start_ampl,end_ampl,fadein_len,fadeout_len,strength]
	sounds[0]=make_sample("p50;[b,2000,1500,1250,50,50,500,1000,2][s,2000,100,100,100,100,500,1000,1]p2050;",0,0,800,50);
	sounds[1]=make_sample("p50;[s,2000,4000,7000,0,100,500,500,1]p1500;[s,1500,7000,8000,0,100,500,500,1]p1650;",0,50,800,100);
	sounds[2]=make_sample("p50;[b,10000,7000,10000,50,100,500,1500,1][s,10000,200,100,50,100,2500,2500,1]p10050;",0,100,800,150);
	sounds[3]=make_sample("p50;[s,2000,8000,5000,50,100,500,1000,1][s,2000,9000,9000,100,100,500,500,2]p2050;",0,150,800,200);
	sounds[4]=make_sample("p50;[s,7500,15000,5000,80,80,500,1500,1]p7550;",0,200,800,250);
	sounds[5]=make_sample("p50;[b,1500,5000,15000,50,50,500,500,1]p1550;",0,250,800,300);
	sounds[6]=make_sample("p100;[s,5000,10000,2000,100,75,500,1000,1]p5100;",0,300,800,350);
	sounds[7]=make_sample("p50;[s,10000,5000,1000,100,75,500,1000,1]p10050;",0,350,800,400);
	sounds[8]=make_sample("p50;[s,10000,2000,8000,75,100,500,1000,1]p10050;",0,400,800,450);
	sounds[9]=make_sample("p100;[s,5000,10000,15000,100,50,250,3250,1]p5100;",0,450,800,500);
	sounds[10]=make_sample("p50;[s,35000,2000,1000,100,80,250,1250,4][b,35000,2500,500,50,25,1250,2250,1]p35050;",0,500,800,550);
	sounds[11]=make_sample("p100;[n,2000,1,1,1,100,250,1250,3][s,2000,200,300,50,1,500,500,1]p2100;",0,450,800,500);

	for (i=0; i<16; i++)
		voice_nr[i]=-1;
}

void play_sample(int sample_nr, int volume, int balance, int speed)
{
	// volume=255, balance=127, speed=1000
     if (sample_nr>=0 && sample_nr<16 && sound_on)
     {
		 //if (voice_nr[sample_nr]==-1 || voice_get_position(voice_nr[sample_nr])<0)
			voice_nr[sample_nr]=play_sample(sounds[sample_nr], volume, balance, speed, FALSE);
     }
}

SAMPLE *make_sample(char *data,int x1, int y1, int x2, int y2)
{	SAMPLE	*newsamp;
	char	*p;
	int		i,i2,length,value,pos,currblock,startf,endf;
	int		starta,enda,fadein,fadeout,len,str,totalblock;
	struct	SOUNDBLOCK sndblock[64];
	double	curramp,multi,addval,ampval;
	int		x,y; //,color;

	//acquire_bitmap(screen);

	for (i=0; i<64; i++) { sndblock[i].length=0; sndblock[i].pos=0; }
	currblock=0;
	// calculate length
	length=0; i=0;
	while (data[i]!=0 && data[i]!='!')
	{
		if (data[i]=='p')
		{	
			i++; value=0;
			while (data[i]!=';')
			{ 
				value*=10; value+=(data[i]-'0'); i++;
			}
			length+=value;
		}
		i++;
	}
	newsamp=create_sample(8, FALSE, 22050, length);
	p = (char *)newsamp->data;
	i=0; pos=0;
	while (data[i]!=0 && data[i]!='!')
	{
		if (data[i]=='[')
		{	// [wave_type,length,start_freq,end_freq,start_ampl,end_ampl,fadein_len,fadeout_len,strength]
			// e.g. [s,1000,2000,2500,100,50,0,0,1]
			i++; sndblock[currblock].wave_type=data[i]; i++;
			i++; len=0; while (data[i]!=',') { len*=10; len+=(data[i]-'0'); i++; }
			i++; startf=0; while (data[i]!=',') { startf*=10; startf+=(data[i]-'0'); i++; }
			i++; endf=0; while (data[i]!=',') { endf*=10; endf+=(data[i]-'0'); i++; }
			i++; starta=0; while (data[i]!=',') { starta*=10; starta+=(data[i]-'0'); i++; }
			i++; enda=0; while (data[i]!=',') { enda*=10; enda+=(data[i]-'0'); i++; }
			i++; fadein=0; while (data[i]!=',') { fadein*=10; fadein+=(data[i]-'0'); i++; }
			i++; fadeout=0; while (data[i]!=',') { fadeout*=10; fadeout+=(data[i]-'0'); i++; }
			i++; str=0; while (data[i]!=']') { str*=10; str+=(data[i]-'0'); i++; }
			sndblock[currblock].length=len;
			sndblock[currblock].pos=0;
			sndblock[currblock].freq=startf;
			sndblock[currblock].freqinc=((double)endf-(double)startf)/(double)len;
			sndblock[currblock].ampl=starta;
			sndblock[currblock].amplinc=((double)enda-(double)starta)/(double)len;
			sndblock[currblock].fadein_len=fadein;
			sndblock[currblock].fadeout_len=fadeout;
			sndblock[currblock].sinpos=0;
			sndblock[currblock].strength=str;
			currblock++; if (currblock>=64) currblock-=64;
		}
		if (data[i]=='p')
		{	// pLENGTH; e.g. p5000;
			i++; value=0;
			while (data[i]!=';') { value*=10; value+=(data[i]-'0'); i++; }
			while (value>0)
			{
				curramp=0; totalblock=0;
				for (i2=0; i2<64; i2++) if (sndblock[i2].pos<sndblock[i2].length)
				{
					multi=1; addval=0;
					if (sndblock[i2].wave_type=='n') // noice
					{
						addval=(rand()%((int)(sndblock[i2].ampl*1000)))/1000;
					}
					if (sndblock[i2].wave_type=='b') // block
					{ 
						if (sndblock[i2].sinpos<M_PI)
							addval=sndblock[i2].ampl; 
						else
							addval=-sndblock[i2].ampl;
					}
					if (sndblock[i2].wave_type=='s') // sine
						addval=sin(sndblock[i2].sinpos)*sndblock[i2].ampl;
					if (sndblock[i2].pos<sndblock[i2].fadein_len)
					{
						multi=(double)sndblock[i2].pos/(double)sndblock[i2].fadein_len;
						multi=sin(multi*M_PI/2);
					}
					if (sndblock[i2].pos>=sndblock[i2].length-sndblock[i2].fadeout_len)
					{
						multi=sndblock[i2].pos-(sndblock[i2].length-sndblock[i2].fadeout_len);
						multi=multi/(double)sndblock[i2].fadeout_len;
						multi=sin((1-multi)*M_PI/2);
					}

					multi*=(double)sndblock[i2].strength;
					curramp+=(addval*multi);
					sndblock[i2].sinpos+=(sndblock[i2].freq/22050);
					while (sndblock[i2].sinpos>=M_PI*2) sndblock[i2].sinpos-=(M_PI*2);
					sndblock[i2].freq+=sndblock[i2].freqinc;
					sndblock[i2].ampl+=sndblock[i2].amplinc;
					sndblock[i2].pos++;
					totalblock+=sndblock[i2].strength;
				}
				if (totalblock>0) curramp/=totalblock;
				if (pos<length)
				{
					ampval=curramp;
					if (pos<100)
						ampval=ampval*pos/100;
					if (pos>=length-100)
						ampval=ampval*(length-pos)/100;

					if (ampval<0)
						p[pos]=(char)-(128+ampval);
					else
						p[pos]=(char)(127-ampval);

					x=x1+(((x2-x1)+1)*pos)/length;
					if (p[pos]>=0)
						y=p[pos];
					else
						y=256+p[pos];
					y=y1+(y*((y2-y1)+1))/256;
					
					//color=getpixel(screen,x,y);
					//color=makecol((getr(color)+getr(makecol(255,255,255)))>>1,(getg(color)+getg(makecol(255,255,255)))>>1,(getb(color)+getb(makecol(255,255,255)))>>1);
					//putpixel(screen,x,y,color);

					pos++;
				}
				value--;
			}
		}
		i++;
	}
    //release_bitmap(screen);

	return newsamp;
}

